#pragma once

#include "IStateNode.h"
#include <string>

namespace OhioState
{
	namespace Graphics 
	{
		enum DrawModeType { SOLID, WIREFRAME, POINTS };
		//
		// Concrete implementation of IStateNode that sets
		// the OpenGL polygon mode (the draw mode).
		//
		class DrawModeNode : public IStateNode
		{
		public:
			//
			// Constructor. All sub-classes require a name.
			//
			DrawModeNode(std::string name, ISceneNode* subject, DrawModeType drawType)
			{
				this->name = name;
				this->subject = subject;
				this->drawType = drawType;
			}
			// 
			// Apply the Draw mode.
			//
			virtual void Apply()
			{
				switch (drawType)
				{
				case SOLID:
					::glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
					break;
				case WIREFRAME:
					::glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
					break;
				case POINTS:
					::glPolygonMode( GL_FRONT_AND_BACK, GL_POINT );
					break;
				}
			}
			// 
			// Remove or undo the application of the draw mode.
			//
			virtual void Unapply()
			{
				::glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
			}
			//
			//
			//
			virtual void AcceptSubject(IVisitor* visitor)
			{
				subject->Accept(visitor);
			}
			void setDrawMode(DrawModeType drawType)
			{
				this->drawType = drawType;
			}
			//
			// Overrrided from ISceneNode
			//
			virtual std::string ToString()
			{
				return name;
			}

		private:
			std::string name;
			ISceneNode* subject;
			DrawModeType drawType;
		};
	}
}