#pragma once

#include "OpenGL.h"

#include "ITransformNode.h"

namespace OhioState
{
	namespace Graphics 
	{
		//
		// A concrete implementation of the ITransformNode.
		// This implementation simply mulitplies the frustum 
		// defined by the current projection transform.
		//
		// This class is immutable.
		//
		class FrustumTransform : public ITransformNode
		{
		public:
			//
			// Constructor. 
			//
			FrustumTransform(std::string name, ISceneNode* subject,
				double left, double right, double bottom, double top, double zNear, double zFar) 
				: ITransformNode(name, subject)
			{
				this->left = left;
				this->right = right;
				this->bottom = bottom;
				this->top = top;
				this->zNear = zNear;
				this->zFar = zFar;
			}
			// 
			// Apply the transform. This implementation will use the OpenGL
			// state machine to track transformations.
			//
			virtual void Apply()
			{
				::glMatrixMode( GL_PROJECTION );
				::glPushMatrix();
				::glFrustum(left, right, bottom, top, zNear, zFar);
			}
			// 
			// Remove or undo the application of the transform. This implementation 
			// will use the OpenGL state machine to track transformations.
			//
			virtual void Unapply()
			{
				::glMatrixMode( GL_PROJECTION );
				::glPopMatrix();
			}
			//
			// Overrrided from ITransformNode
			//
			virtual std::string ToString()
			{
				std::string name = this->ITransformNode::ToString();
				// TODO: add more information about this transform node
				//   to the string.
				return name;
			}

		private:
			double left, right;
			double bottom, top;
			double zNear, zFar;
		};
	}
}