#pragma once

#include "IStateNode.h"
#include "ILight.h"
#include <vector>
#include <string>

namespace OhioState
{
	namespace Graphics 
	{
		//
		// Concrete implementation of IStateNode that allows for
		// a collection of ILights. Enables OpenGL lighting.
		//
		class LightSetNode : public IStateNode
		{
		public:
			//
			// Constructor. All sub-classes require a name.
			//
			LightSetNode(std::string name, ISceneNode* subject)
			{
				this->name = name;
				this->subject = subject;
			}
			// 
			// Turn on the lights.
			//
			virtual void Apply()
			{
				::glEnable( GL_LIGHTING );

				std::vector<ILight*>::iterator lightIterator;
				int lightSlot = 0;
				for( lightIterator = lights.begin(); lightIterator != lights.end(); lightIterator++ )
				{
					(*lightIterator)->Enable(lightSlot);
					lightSlot++;
				}
			}
			// 
			// Turn off this lights.
			//
			virtual void Unapply()
			{
				::glDisable( GL_LIGHTING );

				std::vector<ILight*>::iterator lightIterator;
				int lightSlot = 0;
				for( lightIterator = lights.begin(); lightIterator != lights.end(); lightIterator++ )
				{
					(*lightIterator)->Disable(lightSlot);
					lightSlot++;
				}
			}
			//
			// Traverse the child sub-graph.
			//
			virtual void AcceptSubject(IVisitor* visitor)
			{
				subject->Accept(visitor);
			}
			//
			// Add a light source to this set.
			//
			void AddLight(ILight* light)
			{
				lights.push_back(light);
			}
			//
			// Overrrided from ISceneNode
			//
			virtual std::string ToString()
			{
				return name;
			}

		private:
			std::string name;
			ISceneNode* subject;
			std::vector<ILight*> lights;
		};
	}
}