#pragma once

#include "OpenGL.h"

#include "ITransformNode.h"
#include "Vector.h"

using namespace OhioState::Math;

namespace OhioState
{
	namespace Graphics 
	{
		//
		// A concrete implementation of the ITransformNode.
		// This implementation simply mulitplies the frustum 
		// defined by the current projection transform.
		//
		// This class is immutable.
		//
		class LookAtTransform : public ITransformNode
		{
		public:
			//
			// Constructor. 
			//
			LookAtTransform(std::string name, ISceneNode* subject,
				Vector3& eyePosition, Vector3& centerOfInterest, Vector3& viewUp) 
				: ITransformNode(name, subject)
			{
				this->eyePosition = eyePosition;
				this->centerOfInterest = centerOfInterest;
				this->viewUp = viewUp;
			}
			// 
			// Apply the transform. This implementation will use the OpenGL
			// state machine to track transformations.
			//
			virtual void Apply()
			{
				::glMatrixMode( GL_MODELVIEW );
				::glPushMatrix();
				::gluLookAt(eyePosition.x, eyePosition.y, eyePosition.z,
					centerOfInterest.x, centerOfInterest.y, centerOfInterest.z, 
					viewUp.x, viewUp.y, viewUp.z);
			}
			// 
			// Remove or undo the application of the transform. This implementation 
			// will use the OpenGL state machine to track transformations.
			//
			virtual void Unapply()
			{
				::glMatrixMode( GL_MODELVIEW );
				::glPopMatrix();
			}
			//
			// Overrrided from ITransformNode
			//
			virtual std::string ToString()
			{
				std::string name = this->ITransformNode::ToString();
				// TODO: add more information about this transform node
				//   to the string.
				return name;
			}

		private:
			Vector3 eyePosition;
			Vector3 centerOfInterest;
			Vector3 viewUp;
		};
	}
}