#pragma once
#include "ISceneNode.h"
#include "IVisitor.h"
#include "MaterialManager.h"
#include <string>

namespace OhioState
{
	namespace Graphics 
	{
		//
		// Material node which uses the MaterialManager
		// to access concrete materials through their string-
		// based name.
		//   Extends the ISceneNode with methods to enable and 
		//   disable a material associated with this node.
		//
		class MaterialProxy : public IMaterialNode
		{
		public:
			//
			// Constructor. All sub-classes require a name and materialName
			//   All names should be unique.
			//   All material names should refer to a registered material.
			//
			MaterialProxy(std::string name, std::string materialName)
			{
				this->name = name;
				this->materialName = materialName;
			}
			//
			// Enable the material associated with this IMaterialNode.
			//
			void EnableMaterial()
			{
				MaterialManager::Instance()->EnableMaterial(materialName);
			}
			//
			// Overrrided from ISceneNode
			//
			virtual std::string ToString()
			{
				return name;
			}

		private:
			std::string name;
			std::string materialName;
		};
	}
}