#pragma once

#include "OpenGL.h"

#include "ITransformNode.h"

namespace OhioState
{
	namespace Graphics 
	{
		//
		// A concrete implementation of the ITransformNode.
		// This implementation simply mulitplies the frustum 
		// defined by the current projection transform.
		//
		class PerspectiveTransform : public ITransformNode
		{
		public:
			//
			// Constructor. 
			//
			PerspectiveTransform(std::string name, ISceneNode* subject,
				double fieldOfView, int width, int height, double zNear, double zFar) 
				: ITransformNode(name, subject)
			{
				this->fieldOfView = fieldOfView;
				this->width = width;
				this->height = height;
				this->zNear = zNear;
				this->zFar = zFar;
			}
			void setWidth(int width)
			{
				this->width = width;
			}
			void setHeight(int height)
			{
				this->height = height;
			}
			// 
			// Apply the transform. This implementation will use the OpenGL
			// state machine to track transformations.
			//
			virtual void Apply()
			{
				::glMatrixMode( GL_PROJECTION );
				::glPushMatrix();
				::glViewport(0,0,width, height);
				double aspectRatio = width / (double) height;
				::gluPerspective(fieldOfView, aspectRatio, zNear, zFar);
			}
			// 
			// Remove or undo the application of the transform. This implementation 
			// will use the OpenGL state machine to track transformations.
			//
			virtual void Unapply()
			{
				::glMatrixMode( GL_PROJECTION );
				::glPopMatrix();
			}
			//
			// Overrrided from ITransformNode
			//
			virtual std::string ToString()
			{
				std::string name = this->ITransformNode::ToString();
				// TODO: add more information about this transform node
				//   to the string.
				return name;
			}

		private:
			double fieldOfView;
			int width, height;
			double zNear, zFar;
		};
	}
}