#pragma once

#include "OpenGL.h"

#include "IMaterial.h"
#include "DepthTestedRasterOps.h"
#include "ShaderProgram.h"
#include "Color.h"

namespace OhioState
{
	namespace Graphics 
	{
		//
		// Concrete implementation of IMaterial.
		//   - Set all fragments to a solid color.
		//   - standard depth testing
		//
		class SimpleShaderMaterial : public IMaterial
		{
		public:
			//
			// Constructor.
			//
			SimpleShaderMaterial(std::string vertexFile, std::string fragmentFile)
			{
				FragmentRoutine* fragRoutine = new FragmentRoutine(fragmentFile);
				VertexRoutine* vertRoutine = new VertexRoutine(vertexFile);
				this->shaderProgram = new ShaderProgram(vertRoutine, fragRoutine);
				this->rasterOps = new DepthTestedRasterOps();
				color = White;
			}
			//
			// Enable the material.
			//
			virtual void Enable()
			{
				::glColor4f( color.r, color.g, color.b, color.a );
				::glPushAttrib(GL_ENABLE_BIT);
				::glDisable(GL_LIGHTING);
				shaderProgram->Enable();
				rasterOps->Enable();
			}
			//
			// Disable the material. This will be a no-op for our system, since
			// we will require these to be set at the beginning of every material
			// when it is enabled.
			//
			virtual void Disable()
			{
				::glPopAttrib();
			}
			//
			// Get the color.
			//
			Color& getColor() const
			{
				return Color(color);
			}
			//
			// Set the color.
			//
			void setColor(Color color)
			{
				this->color = color;
			}

		private:
			IRasterOps* rasterOps;
			IShaderProgram* shaderProgram;
			Color color;
		};
	}
}