void PointLight(in int i, in vec3 eye, in vec3 ecPosition3, in vec3 normal,
		inout vec4 ambient, inout vec4 diffuse, inout vec4 specular)
{
	float nDotVP;	// normal . light direction
	float nDotHV;	// normal . half vector
	float pf;		// power factor
	float attenuation;	// computed attenuation factor
	float d;			// distance from surface to light source
	vec3 VP;			// direction from surface to light position
	vec3 halfVector;	// direction of maximum highlights
	
	// Compute vector from surface to light position
	VP = vec3(gl_LightSource[i].position) - ecPosition3;
	
	// Compute distance between surface and light position
	d = length(VP);
	
	// Compute attenuation
	attenuation = 1.0 / (gl_LightSource[i].constantAttenuation
				+ gl_LightSource[i].linearAttenuation * d
				+ gl_LightSource[i].quadraticAttenuation * d * d);
	
	halfVector = normalize(VP + eye);
	
	nDotVP = max(0.0, dot(normal,normalize(VP)));
	nDotHV = max(0.0, dot(normal, halfVector));
	
	if( nDotVP == 0.0)
		pf = 0.0;
	else
		pf = pow(nDotHV, gl_FrontMaterial.shininess);
	
	ambient		+= gl_LightSource[i].ambient * attenuation;
	diffuse		+= gl_LightSource[i].diffuse * nDotVP * attenuation;
	specular	+= gl_LightSource[i].specular * pf * attenuation;
}


void DirectionalLight(in int i, in vec3 normal,
		inout vec4 ambient, inout vec4 diffuse, inout vec4 specular)
{
	float nDotVP;	// normal . light direction
	float nDotHV;	// normal . half vector
	float pf;		// power factor
	
	nDotVP = max(0.0, dot(normal, normalize(vec3(gl_LightSource[i].position))));
	nDotHV = max(0.0, dot(normal, vec3(gl_LightSource[i].halfVector)));
	
	if( nDotVP == 0.0)
		pf = 0.0;
	else
		pf = pow(nDotHV, gl_FrontMaterial.shininess);
	
	ambient		+= gl_LightSource[i].ambient;
	diffuse		+= gl_LightSource[i].diffuse * nDotVP;
	specular	+= gl_LightSource[i].specular * pf;
}

void main()
{
	vec4 ambient = vec4(0.0,0.0,0.0,0.0);
	vec4 diffuse = vec4(0.0,0.0,0.0,0.0);
	vec4 specular = vec4(0.0,0.0,0.0,0.0);
	
	// Calculate the eye position
	vec4 ecPosition = gl_ModelViewMatrix * gl_Vertex;
	vec3 ecPosition3 = (vec3(ecPosition)) / ecPosition.w;
	vec3 eye = vec3(0,0,1);
    // transform vertex to clip space coordinates
    gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
    // Calculate the lighting
    vec3 normal = normalize(gl_NormalMatrix * gl_Normal);
    for(int i=0; i <2; i++)
    {
    	if( gl_LightSource[i].position.w <= 0.1)
    		DirectionalLight(i, normal, ambient, diffuse, specular);
    	else
    		PointLight(i, eye, ecPosition3, normal, ambient, diffuse, specular);
    }
    // Set the color
    specular = specular * gl_FrontMaterial.specular;
    vec4 color = gl_FrontLightModelProduct.sceneColor + ambient * gl_FrontMaterial.ambient
    			+ diffuse * gl_FrontMaterial.diffuse
    			+ specular;
    gl_FrontColor = color;
}



